package tareas.tarea2.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import tareas.tarea2.electronica.Componente;
import tareas.tarea2.electronica.Nodo;
import tareas.tarea2.electronica.Cable;

/**
 * Clase que implementa los metodos, que trasladan un componente, pedidos en la
 * tarea.
 * @author Klaus
 */
public class DestroyListener implements ActionListener, MouseListener,
        MouseMotionListener {

    /**
     * Panel donde se efectuaran los cambios mediante el mouse.
     */
    private Dibujo dibujo;
    /**
     * Forma a editarse.
     */
    private Componente componente;
    /**
     * ID de la forma seleccionada.
     */
    private int selected;

    /**
     * Constructor de la clase.
     * @param dib Panel donde se efectuaran los cambios mediante el mouse.
     */
    public DestroyListener(Dibujo dib) {
        dibujo = dib;
        componente = null;
    }

    /**
     * Metodo que se ejectuta cuando se dispara el evento ActionEvent.
     * @param event Evento de accion que pertenece al objeto al cual se le
     * asigno este Listener.
     */
    public void actionPerformed(ActionEvent event) {
        dibujo.changeMouseListener(this);
        dibujo.changeMouseMotionListener(this);
        dibujo.repaint();
    }

    /**
     * Metodo que se ejectuta cuando se presiona el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mousePressed(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando se suelta el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseReleased(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando se hace click.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseClicked(MouseEvent event) {
        if (selected >= 0) {
            componente = dibujo.getComponente(selected);
            if (componente instanceof Cable) {
                ((Cable) componente).destroy();
            } else {
                Nodo[] nodos = componente.getNodos();
                for (int i = 0; i < nodos.length; i++) {
                    ArrayList < Cable > cables = nodos[i].getCables();
                    while (cables.size() > 0) {
                        dibujo.removeComponente(cables.get(0));
                        cables.get(0).destroy();
                    }
                }
            }
            dibujo.removeComponente(componente);
            dibujo.repaint();
        }
    }

    /**
     * Metodo que se ejectuta cuando el mouse entra en el dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseEntered(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse sale del dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseExited(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseDragged(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseMoved(MouseEvent event) {
        selected = dibujo.selectedComponente(event.getPoint());
        dibujo.repaint();
    }
}
