package tareas.tarea2.gui;

import java.awt.Color;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JPanel;
import java.util.ArrayList;
import tareas.tarea2.electronica.Componente;
import tareas.tarea2.electronica.Circuito;
import tareas.tarea2.electronica.Nodo;

/** La clase Dibujo es el panel donde estaran las formas.
 */
public class Dibujo extends JPanel {
    /**
     * Arreglo de formas que se dibujaran.
     */
    private Circuito circuito;
    /**
     * Listener que escucha los botones del mouse.
     */
    private MouseListener currentMouseListener;
    /**
     * Listener que escucha el movimiento del mouse.
     */
    private MouseMotionListener currentMouseMotionListener;
    /**
     * Numero Serial de la clase.
     */
    private static final long serialVersionUID = 1;

    /** Crea una nueva instancia de Dibujo.
     */
    public Dibujo() {
        super();
        circuito = new Circuito();
        DefaultListener defaultListener = new DefaultListener(this);
        currentMouseListener = defaultListener;
        currentMouseMotionListener = defaultListener;
        addMouseListener(currentMouseListener);
        addMouseMotionListener(currentMouseMotionListener);
    }

    /** Metodo que pinta todas las formas en el panel.
     * @param g objeto grafico que nos permite dibujar.
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;
        Color color = Color.WHITE;
        g2d.setColor(color);
        g2d.fill(new Rectangle(0, 0, getWidth(), getHeight()));
        circuito.paintComponent(g2d);
    }

    /** Metodo que escucha los botones del mouse que solo permite
     * que 1 metodo escuche a este.
     * @param l objeto que permite escuchar al mouse.
     */
    public void changeMouseListener(MouseListener l) {
        if (currentMouseListener != null) {
            removeMouseListener(currentMouseListener);
        }
        currentMouseListener = l;
        addMouseListener(currentMouseListener);
    }

    /** Metodo que escucha el movimiento del mouse que solo permite
     * que 1 metodo escuche a este.
     * @param l objeto que permite escuchar al mouse.
     */
    public void changeMouseMotionListener(MouseMotionListener l) {
        if (currentMouseMotionListener != null) {
            removeMouseMotionListener(currentMouseMotionListener);
        }
        currentMouseMotionListener = l;
        addMouseMotionListener(currentMouseMotionListener);
    }

    /** Quita una forma del panel.
     * @param f La forma a quitar.
     */
    public void removeComponente(Componente f) {
        circuito.removeForma(f);
        repaint();
    }
    /** Aade una forma del panel.
     * @param f La forma a aadir.
     */
    public void addComponente(Componente f) {
        circuito.addForma(f);
        repaint();
    }

    /** Traslada todas las formas seleccionadas.
     * @param x Distancia a trasladarse en el eje "x"
     * @param y Distancia a trasladarse en el eje "y"
     */
    public void trasladaDibujo(double x, double y) {
        circuito.trasladar(x, y);
        repaint();
    }

    /** Rota todas las formas seleccionadas.
     */
    public void rotaDibujo() {
        circuito.rotar();
        repaint();
    }

    /**
     * Obtiene el ID de la forma cuya area contine el punto p.
     * @param p Punto a verificar.
     * @return El ID de la forma. -1 si no hubo coincidencia.
     */
    public int selectedComponente(Point p) {
        return circuito.selectedForma(p);
    }

    /** Retorna una forma del panel.
     * @param i ID de la forma.
     * @return La instancia de forma pedida.
     */
    public Componente getComponente(int i) {
        return ((Componente) circuito.getForma(i));
    }

    /** Retorna un arreglo de nodo menos los que pertenescan al componente con
     * ID i.
     * @param i ID de la forma.
     * @return arreglo de nodos.
     */
    public ArrayList < Nodo > getNodos(int i) {
        return circuito.getNodos(i);
    }
}
