package tareas.tarea2.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.Point;
import java.util.ArrayList;
import tareas.tarea2.electronica.Cable;
import tareas.tarea2.electronica.Componente;
import tareas.tarea2.electronica.Nodo;

/**
 * Clase que implementa los metodos, que trasladan un componente, pedidos en la
 * tarea.
 * @author Klaus
 */
public class TrasladarListener implements ActionListener, MouseListener,
        MouseMotionListener {

    /**
     * Panel donde se efectuaran los cambios mediante el mouse.
     */
    private Dibujo dibujo;
    /**
     * Estado del proceso.
     */
    private int estado;
    /**
     * Punto anterior usado para calcual el deltaX y el deltaY.
     */
    private Point previo;
    /**
     * Forma a editarse.
     */
    private Componente componente;
    /**
     * Forma a editarse en caso de tener una posible union.
     */
    private Componente componenteClon;
    /**
     * Nodo que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo1;
    /**
     * Nodo clonado que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo2;
    /**
     * Nodo que se fijara en caso de tener una posible union.
     */
    private Nodo nodoFijo3;
    /**
     * Valor del estado al inicio.
     */
    private static final int INICIO = 0;
    /**
     * Valor del estado una vez que se ha seleccionado la figura.
     */
    private static final int SELECTED = 1;
    /**
     * Valor del estado una vez que se ha encontrado una posible union.
     */
    private static final int UNITED = 2;
    /**
     * ID de la forma seleccionada.
     */
    private int selected;

    /**
     * Constructor de la clase.
     * @param dib Panel donde se efectuaran los cambios mediante el mouse.
     */
    public TrasladarListener(Dibujo dib) {
        dibujo = dib;
        componente = null;
        componenteClon = null;
        nodoFijo1 = null;
        nodoFijo2 = null;
        nodoFijo3 = null;
        selected = -1;
        estado = INICIO;
    }

    /**
     * Metodo que se ejectuta cuando se dispara el evento ActionEvent.
     * @param event Evento de accion que pertenece al objeto al cual se le
     * asigno este Listener.
     */
    public void actionPerformed(ActionEvent event) {
        dibujo.changeMouseListener(this);
        dibujo.changeMouseMotionListener(this);
        dibujo.repaint();
    }

    /**
     * Metodo que se ejectuta cuando se presiona el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mousePressed(MouseEvent event) {
        if (selected >= 0) {
            estado = SELECTED;
            previo = event.getPoint();
            componente = dibujo.getComponente(selected);
        }
    }

    /**
     * Metodo que se ejectuta cuando se suelta el mouse.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseReleased(MouseEvent event) {
        if (estado == UNITED) {
            dibujo.addComponente(new Cable(nodoFijo1, nodoFijo3));
        }
        estado = INICIO;
        componente = null;
        componenteClon = null;
        nodoFijo1 = null;
        nodoFijo2 = null;
        nodoFijo3 = null;
        selected = -1;
        dibujo.repaint();
    }

    /**
     * Metodo que se ejectuta cuando se hace click.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseClicked(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse entra en el dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseEntered(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse sale del dibujo.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseExited(MouseEvent event) { }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseDragged(MouseEvent event) {
        if (estado == SELECTED) {
            Point p = event.getPoint();
            double deltaX = p.getX() - previo.getX();
            double deltaY = p.getY() - previo.getY();
            componente.trasladar(deltaX, deltaY);
            dibujo.repaint();
            previo = p;
            Nodo[] n1 = componente.getNodos();
            ArrayList < Nodo > n2 = dibujo.getNodos(selected);
            for (int i = 0; i < n2.size(); i++) {
                for (int j = 0; j < n1.length; j++) {
                    if (n2.get(i).intersects(n1[j])) {
                        deltaX = n2.get(i).getPunto().getX()
                            - n1[j].getPunto().getX();
                        deltaY = n2.get(i).getPunto().getY()
                            - n1[j].getPunto().getY();
                        nodoFijo1 = n2.get(i);
                        componenteClon = (Componente) componente.clone();
                        nodoFijo2 = componenteClon.getNodos()[j];
                        nodoFijo3 = n1[j];
                        componente.trasladar(deltaX, deltaY);
                        estado = UNITED;
                    }
                }
            }
        }
        if (estado == UNITED) {
            Point p = event.getPoint();
            double deltaX = p.getX() - previo.getX();
            double deltaY = p.getY() - previo.getY();
            componenteClon.trasladar(deltaX, deltaY);
            previo = p;
            if (!nodoFijo1.intersects(nodoFijo2)) {
                deltaX = componenteClon.getPunto().getX()
                    - componente.getPunto().getX();
                deltaY = componenteClon.getPunto().getY()
                    - componente.getPunto().getY();
                componente.trasladar(deltaX, deltaY);
                estado = SELECTED;
            }
        }
    }

    /**
     * Metodo que se ejectuta cuando el mouse se arrastra una vez presionado.
     * @param event Evento de mouse que tiene encapsulado las variables del
     * mouse.
     */
    public void mouseMoved(MouseEvent event) {
        selected = dibujo.selectedComponente(event.getPoint());
        dibujo.repaint();
    }
}
