package tareas.tarea3.electronica;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import tareas.tarea2.electronica.Componente;
import tareas.tarea2.electronica.Nodo;

/**
 * Clase hija de Componente que implementa una fuente de voltaje.
 * @author Janther
 */
public class FuenteAC extends Componente {

    /**
     * Numero de nodos en el Componente.
     */
    private final int numNodos = 2;
    /**
     * Nodos del Componente.
     */
    private Nodo[] nodos = new Nodo[numNodos];
    /**
     * Numero de vertices en la Forma.
     */
    private final int numVertices = 4;
    /**
     * Vertices del Componente.
     */
    private Point[] vertices = new Point[numVertices];
    /**
     * Tamao del Componente.
     */
    private final double size = 36;
    /**
     * Tamao del Circulo.
     */
    private final double sizeCir = 24;
    /**
     * El area para seleccionar el componente.
     */
    private Rectangle2D area;
    /**
     * Amplitud de la fuente.
     */
    private double amplitud = 1;
    /**
     * Frecuencia de la fuente.
     */
    private double frecuencia = 1;
    /**
     * Color de la forma.
     */
    private Color c;
    /**
     * Circulo de la representacion del componente.
     */
    private Ellipse2D circulo = new Ellipse2D.Double();
    /**
     * Primera mitad de la sinusoide.
     */
    private Arc2D sin0 = new Arc2D.Double();
    /**
     * Segunda mitad de la sinusoide.
     */
    private Arc2D sin1 = new Arc2D.Double();

    /**
     * Constructor de la clase que la dibuja en la esquina superior del panel.
     * @param a Amplitud de la fuente.
     * @param f Frecuencia de la fuente.
     */
    public FuenteAC(double a, double f) {
        this(new Point());
        trasladar(0, sizeCir / 2);
        amplitud = a;
        frecuencia = f;
    }

    /**
     * Constructor de la clase que la dibuja en la esquina superior del panel.
     * @param inicio Punto donde se encuentra el primer punto del dibujo del
     * componente.
     */
    public FuenteAC(Point inicio) {
        double x = inicio.getX();
        double y = inicio.getY();
        for (int i = 0; i < numVertices; i++) {
            vertices[i] = new Point();
        }
        final int maxDiv = 12;
        vertices[0].setLocation(x, y);
        vertices[1].setLocation(x + size * 2 / maxDiv, y);
        vertices[2].setLocation(x + size * 10 / maxDiv, y);
        vertices[3].setLocation(x + size, y);
        circulo.setFrame(
                vertices[1].getX(),
                y - sizeCir / 2,
                sizeCir,
                sizeCir);
        sin0.setFrame(
                circulo.getX() + sizeCir / 4,
                circulo.getY() + sizeCir / 4,
                sizeCir / 4,
                sizeCir / 2);
        sin0.setAngleExtent(180);
        sin0.setAngleStart(0);

        sin1.setFrame(
                circulo.getX() + sizeCir / 2,
                circulo.getY() + sizeCir / 4,
                sizeCir / 4,
                sizeCir / 2);
        sin1.setAngleExtent(180);
        sin1.setAngleStart(180);

        nodos[0] = new Nodo(vertices[0]);
        nodos[1] = new Nodo(vertices[numVertices - 1]);
        setArea();
    }

    /**
     * Constructor de la clase que sirve para clonar.
     * @param esquinas Esquinas que dibujaran la fuente.
     * @param a Amplitud de la fuente.
     * @param f Frecuencia de la fuente.
     * @param cir area donde se dibujara el circulo de la representacion
     * grfica de la fuente.
     */
    public FuenteAC(Point[] esquinas, double a, double f, Rectangle2D cir) {
        for (int i = 0; i < esquinas.length; i++) {
            vertices[i] = (Point) esquinas[i].clone();
        }
        frecuencia = f;
        circulo.setFrame(cir);
        sin0.setFrame(
                circulo.getX() + sizeCir / 4,
                circulo.getY() + sizeCir / 4,
                sizeCir / 4,
                sizeCir / 2);
        sin0.setAngleExtent(180);
        sin0.setAngleStart(0);

        sin1.setFrame(
                circulo.getX() + sizeCir / 2,
                circulo.getY() + sizeCir / 4,
                sizeCir / 4,
                sizeCir / 2);
        sin1.setAngleExtent(180);
        sin1.setAngleStart(180);

        amplitud = a;
        nodos[0] = new Nodo(vertices[0]);
        nodos[1] = new Nodo(vertices[numVertices - 1]);
        setArea();
    }

    /**
     * Setea el area seleccionable del componente en base a los puntos que lo
     * conforman.
     */
    private void setArea() {
        double maxX = vertices[0].getX();
        double maxY = vertices[0].getY();
        double minX = vertices[0].getX();
        double minY = vertices[0].getY();
        for (int i = 1; i < numVertices; i++) {
            if (vertices[i].getX() < minX) {
                minX = vertices[i].getX();
            }
            if (vertices[i].getX() > maxX) {
                maxX = vertices[i].getX();
            }
            if (vertices[i].getY() < minY) {
                minY = vertices[i].getY();
            }
            if (vertices[i].getY() > maxY) {
                maxY = vertices[i].getY();
            }
        }
        double ancho = maxX - minX + 1;
        double alto = maxY - minY + 1;
        if (ancho == 1) {
            ancho = sizeCir + 1;
            minX = minX - sizeCir / 2;
        } else if (alto == 1) {
            alto = sizeCir + 1;
            minY = minY - sizeCir / 2;
        }
        area = new Rectangle2D.Double(minX, minY, ancho, alto);
    }

    /** Escala la forma respecto al punto central en un factor dado.
     * @param centro Punto respecto del cual se hace el escalamiento.
     * @param factor  Factor usado en el escalamiento. >1 corresponde
     * a amplificacion, <1 corresponde a reduccion.
     */
    public void escalar(Point centro, double factor) {
        /*
         * Se traslado cada punto de la Fuente con respecto al punto
         * "centro" en funcion del factor.
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * factor + centro.getX();
            ny = (vertices[i].getY() - centro.getY()) * factor + centro.getY();
            vertices[i].setLocation(nx, ny);
        }
        for (int i = 0; i < numNodos; i++) {
            nodos[i].escalar(centro, factor);
        }
        setArea();
    }

    /** Rota la forma respecto al punto central en un �ngulo dado.
     * @param centro Punto respecto del cual se hace la rotaci�n.
     * @param deltaAngulo angulo de rotacion en radianes.
     */
    public void rotar(Point centro, double deltaAngulo) {
        /*
         * Se uso la transformacion rotacional para un vector en cada punto.
         * http://es.wikipedia.org/wiki/Rotaci%C3%B3n
         */
        double nx, ny;
        for (int i = 0; i < numVertices; i++) {
            nx = (vertices[i].getX() - centro.getX()) * Math.cos(deltaAngulo)
                - (vertices[i].getY() - centro.getY()) * Math.sin(deltaAngulo);
            ny = (vertices[i].getX() - centro.getX()) * Math.sin(deltaAngulo)
                + (vertices[i].getY() - centro.getY()) * Math.cos(deltaAngulo);
            vertices[i].setLocation(nx + centro.getX(), ny + centro.getY());
        }
        for (int i = 0; i < numNodos; i++) {
            nodos[i].rotar(centro, deltaAngulo);
        }
        setArea();
    }

    /** Rota la forma respecto a su centro en 90 grados.
     */
    public void rotar() {
        double nx = area.getX() + (area.getWidth() - 1) / 2;
        double ny = area.getY() + (area.getHeight() - 1) / 2;
        Point centro = new Point();
        centro.setLocation(nx, ny);
        rotar(centro, Math.PI / 2);
    }

    /** Traslada la forma desplazandola en sentido X e Y.
     * @param deltaX desplazamiento en sentido horizontal.
     * @param deltaY desplazamiento en sentido vertical.
     */
    public void trasladar(double deltaX, double deltaY) {
        /*
         * Se translado cada coordenada en su delta respectivo.
         */
        for (int i = 0; i < numVertices; i++) {
            vertices[i].setLocation(vertices[i].getX() + deltaX,
                                    vertices[i].getY() + deltaY);
        }
        for (int i = 0; i < numNodos; i++) {
            nodos[i].trasladar(deltaX, deltaY);
        }
        circulo.setFrame(
                circulo.getX() + deltaX,
                circulo.getY() + deltaY,
                sizeCir,
                sizeCir);
        sin0.setFrame(
                sin0.getX() + deltaX,
                sin0.getY() + deltaY,
                sin0.getWidth(),
                sin0.getHeight());
        sin1.setFrame(
                sin1.getX() + deltaX,
                sin1.getY() + deltaY,
                sin1.getWidth(),
                sin1.getHeight());
        setArea();
    }

    /**
     * Verifica si un punto esta dentro de el area seleccionable de la forma.
     * @param p Punto a verificar.
     * @return Verdadero si el punto esta dentro del area.
     */
    public boolean isSelected(Point p) {
        if (area.contains(p) || nodos[0].isSelected(p)
                || nodos[1].isSelected(p)) {
            c = Color.RED;
            return true;
        }
        c = Color.BLACK;
        return false;
    }

    /**
     * Abre una ventana de dialogo para editar el valor de la componente.
     */
    public void edit() {
        Object[] message = new Object[4];
        //Primer objeto es un Label.
        message[0] = "Valor de la Amplitud:";
        //El segundo objeto es un Panel con las opciones.
        JPanel panel = new JPanel();
        JSpinner a =
            new JSpinner(new SpinnerNumberModel(amplitud, null, null, 1));
        a.setPreferredSize(new Dimension(150, 20));
        panel.add(a);
        panel.add(new JLabel("[V]"));
        message[1] = panel;
        //Tercer objeto es un Label.
        message[2] = "Valor de la Frecuencia:";
        //El cuarto objeto es un Panel con las opciones.
        JPanel panel2 = new JPanel();
        panel2.setSize(100, 20);
        JSpinner f =
            new JSpinner(new SpinnerNumberModel(frecuencia, null, null, 1));
        f.setPreferredSize(new Dimension(150, 20));
        panel2.add(f);
        panel2.add(new JLabel("[H]"));
        message[3] = panel2;
        String[] options = {
                "Aceptar",
                "Cancelar"
        };
        //Se instancia la ventana.
        int result = JOptionPane.showOptionDialog(
                null,
                message,
                "Editor de Fuente Alterna",
                JOptionPane.DEFAULT_OPTION,
                JOptionPane.INFORMATION_MESSAGE,
                new ImageIcon(getClass().getResource(
                        "/iconos/FuenteAC.jpg")),
                options,
                options[0]
        );
        switch(result) {
        case 0:
            //Opcion Aceptar
            amplitud = (Double) a.getValue();
            frecuencia = (Double) f.getValue();
            break;
        case 1:
            //Opcion Cancelar
            break;
        default:
            break;
        }
    }

    /** Dibuja la Forma en el Graphics2D.
     * @param g2d Grafico en el cual se incluira el dibujo.
     */
    public void paintComponent(Graphics2D g2d) {
        g2d.setColor(c);
        for (int i = 1; i < numVertices; i = i + 2) {
            g2d.draw(new Line2D.Double(vertices[i - 1], vertices[i]));
        }
        g2d.draw(circulo);
        g2d.draw(sin0);
        g2d.draw(sin1);

        for (int i = 0; i < numNodos; i++) {
            nodos[i].paintComponent(g2d);
        }
    }

    /** Retorna los nodos relacionados a este Componente.
     * @return Arreglo de nodos.
     */
    public Nodo[] getNodos() {
        return nodos;
    }

    /** Redefine el metodo clone.
     * @return Un Objeto clon de este.
     */
    public Object clone() {
        return new FuenteAC(
                vertices,
                amplitud,
                frecuencia,
                circulo.getFrame());
    }

    /** Retorna el punto que basta para localizar la coordenada de un
     * componente, no asi su orientacion.
     * @return Punto con las coordenadas del componente.
     */
    public Point getPunto() {
        return vertices[0];
    }
}
